package middleware

import (
	"context"

	"git.perx.ru/perxis/perxis-go/pkg/cache"
	"git.perx.ru/perxis/perxis-go/pkg/data"
	service "git.perx.ru/perxis/perxis-go/pkg/locales"
)

func CachingMiddleware(cache cache.Cache) Middleware {
	return func(next service.Locales) service.Locales {
		return &cachingMiddleware{
			cache: cache,
			next:  next,
		}
	}
}

type cachingMiddleware struct {
	cache cache.Cache
	next  service.Locales
}

func (m cachingMiddleware) Create(ctx context.Context, locale *service.Locale) (loc *service.Locale, err error) {

	loc, err = m.next.Create(ctx, locale)
	if err == nil {
		_ = m.cache.Remove(loc.SpaceID)
	}
	return loc, err
}

func (m cachingMiddleware) Update(ctx context.Context, locale *service.Locale) (err error) {

	err = m.next.Update(ctx, locale)
	if err == nil {
		_ = m.cache.Remove(locale.SpaceID)
	}
	return err
}

func (m cachingMiddleware) List(ctx context.Context, spaceId string) (locales []*service.Locale, err error) {

	value, e := m.cache.Get(spaceId)
	if e == nil {
		return data.CloneSlice(value.([]*service.Locale)), nil
	}
	locales, err = m.next.List(ctx, spaceId)
	if err == nil {
		_ = m.cache.Set(spaceId, locales)
		return data.CloneSlice(locales), nil
	}
	return nil, err
}

func (m cachingMiddleware) Delete(ctx context.Context, spaceId string, localeId string) (err error) {

	err = m.next.Delete(ctx, spaceId, localeId)
	if err == nil {
		_ = m.cache.Remove(spaceId)
	}
	return err
}
