package yaml

import (
	"gopkg.in/yaml.v3"
)

var (
	NewDecoder = yaml.NewDecoder
	NewEncoder = yaml.NewEncoder
	Unmarshal  = yaml.Unmarshal
	Marshal    = yaml.Marshal
)

var tagResolvers = make(map[string]Resolver)

func RegisterTagResolver(tag string, resolver Resolver) {
	tagResolvers[tag] = resolver
}

// Resolver обрабатывает тег YAML и возвращает его обработанный вариант
type Resolver interface {
	Resolve(tp TagProcessor, node *yaml.Node) (*yaml.Node, error)
}

type ResolverFunc func(TagProcessor, *yaml.Node) (*yaml.Node, error)

func (fn ResolverFunc) Resolve(tp TagProcessor, node *yaml.Node) (*yaml.Node, error) {
	return fn(tp, node)
}

func init() {
	RegisterTagResolver("!include", ResolverFunc(IncludeResolver))
	RegisterTagResolver("!file", ResolverFunc(FileResolver))
}
